; This code is a ParaSys boot stub - it asks Zeus to provide the slave code, loads it into memory and runs it.

; The UART

;Zeus_parasys_baud       = 2000000                       ; Override the baudrate set by the Zeus IDE (unless commented out)

UART_RxD                equ $143B                       ; Also used to set the baudrate
UART_TxD                equ $133B                       ; Also reads status
UART_SetBaud            equ UART_RxD                    ; Sets baudrate
UART_GetStatus          equ UART_TxD                    ; Reads status bits

; Status bit masks

UART_mRX_DATA_READY     equ %xxxxx 0 0 1                ;
UART_mTX_BUSY           equ %xxxxx 0 1 0                ;
UART_mRX_FIFO_FULL      equ %xxxxx 1 0 0                ;

; Protocol characters

cpcSTUFF                equ $A6                         ; PSSI protocol bytes
cpcSOM                  equ $A7                         ;

; Call this once to set the UART up before we can use it

BootTestSetup           proc                            ;

                        ld bc,$243B                     ; Now adjust for the set Video timing.
                        ld a,17                         ;
                        out (c),a                       ;

                        inc b                           ;
                        in a,(c)                        ;

                        add a,a                         ; 2 bytes per entry

                        ld hl,BaudTable                 ;
                        add hl,a                        ;
                        ld a,(hl)                       ;

                        ld bc,UART_SetBaud              ;

                        ld a,(hl)                       ;
                        out(c),a                        ;

                        inc hl

                        ld a,(hl)                       ;
                        out(c),a                        ;

                        ret                             ; Done

; This plants a baud-rate table with a byte for each of the seven possible video timing modes

; We're going to define a function to calculate the value required to set the required baud rate given a video mode. Shitty fucking hardware.

[[
function CalcBaudRate(videomode,baudrate)
  {
  // Get the clock-speed
  if videomode = 0
    cpu_clk=28000000
  else if videomode = 1
    cpu_clk=28571429
  else if videomode = 2
    cpu_clk=29464286
  else if videomode = 3
    cpu_clk=30000000
  else if videomode = 4
    cpu_clk=31000000
  else if videomode = 5
    cpu_clk=32000000
  else if videomode = 6
    cpu_clk=33000000
  else
    cpu_clk=27000000
  // Now, how many "ticks" are needed for any given baudrate?
  Ticks=cpu_clk/baudrate

  // Now, we have to fuck about with this value and store it as a horribly-mangled 16-bit number

  firstbyte  =Ticks        and %0111 1111
  secondbyte =(Ticks >> 7) or  %1000 0000

  result=firstbyte or (secondbyte<<8)
  }
]]

BaudTable               equ *                           ; Base of table

                        for vm=0 to 7                   ; Iterate through the eight possible clock speeds
                          dw CalcBaudRate(vm,Zeus_parasys_baud) ; Calc the baud-rate
                          next                          ;

                        pend                            ;

; Call this periodically and it will "install" ParaSys when there is any comms activity

BootTest                proc                            ; Hide all the labels

                        ld a,high UART_GetStatus        ; Are there any characters waiting?
                        in a,(low UART_GetStatus)       ; This inputs from the 16-bit address UART_GetStatus
                        and UART_mRX_DATA_READY         ; We're just interested in the DATA_READY flag
                        ret z                           ; And we return if there isn't any.

                        call BootPara.BootRxD           ; See what it is

                        cp cpcSOM                       ; Start of a message
                        ret nz                          ; Nope

                        ; There are characters in the buffer, assume Zeus wants to talk to us...

        if (BootPara - *) > 0                           ; Jump to BootPara unless we can just fall into it...
          jp BootPara                                   ;
        endif                                           ;

                        pend

BootPara                proc                            ; Hide all the labels

                        ; Now, there's something waiting, so we want to boot ParaSys...

                        di                              ; Kill the interrupts

                        ; We need to tell Zeus we want the code and where it will go.
                        ld hl,BootParaBase              ; Put it here

                        ld d,cpcSTUFF                   ; Tell Zeus to give us the ParaSys code
                        call BootTxD_D                  ;
                        ld d,$FF                        ; STUFF+$FF is the request for the boot code
                        call BootTxD_D                  ;
                        ld d,l                          ; Followed by the base address for where it's to go
                        call BootTxD_D                  ;
                        ld d,h                          ;
                        call BootTxD_D                  ;

                        ; Now wait for the start of the boot code (There will be crap in the buffer)

BootWait                call BootRxD                    ; Get a char
                        cp cpcSTUFF                     ; STUFF?
                        jr nz BootWait                  ; Nope
                        call BootRxD                    ; Get a char
                        inc a                           ; Was it $FF?
                        jr nz BootWait                  ; Nope

                        ; We now expect the terminator code (it can change)

                        call BootRxD                    ; Get the terminator byte
                        ld e,a                          ;

BootPlantLp             ld a,high UART_GetStatus        ; Are there any characters waiting?
                        in a,(low UART_GetStatus)       ; This inputs from the 16-bit address UART_GetStatus
        if UART_mRX_DATA_READY = $01
                        rrca                            ; Test bit zero
                        jr nc BootPlantLp               ; Not ready yet
        else
                        and UART_mRX_DATA_READY         ; We're just interested in the DATA_READY flag
                        jr z BootPlantLp                ; Nothing
        endif
                        ; Read it

                        ld a,high UART_RxD              ; Read it
                        in a,(low UART_RxD)             ;

                        ld (hl),a                       ; Store it
                        inc hl                          ;

                        cp e                            ; End?
                        jr nz BootPlantLp               ; No, loop

                        jp BootParaBase                 ; Start the ParaSys driver

; Read a byte

BootRxD                 ld a,high UART_GetStatus        ; Are there any characters waiting?
                        in a,(low UART_GetStatus)       ; This inputs from the 16-bit address UART_GetStatus
        if UART_mRX_DATA_READY = $01
                        rrca                            ; Test bit zero
                        jr nc BootRxD                   ; Not ready yet
        else
                        and UART_mRX_DATA_READY         ; We're just interested in the DATA_READY flag
                        jr z BootRxD                    ; Nothing
        endif
                        ; Read it

                        ld a,high UART_RxD              ; Read it
                        in a,(low UART_RxD)             ;

                        ret                             ; Done

; Send a byte

BootTxD_D               ld bc,UART_GetStatus            ; Send a byte

BootTxDWait             in a,(c)                        ; Wait until it's sent
                        and UART_mTX_BUSY               ;
                        jr nz BootTxDWait               ; Loop

                        out (c),d                       ; Send it

                        ret                             ; Done

                        pend                            ; And we're done


